# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models.external_url import ExternalUrl  # noqa: F401,E501
from odahuflow.sdk.models import util


class CommonConfiguration(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, external_urls: List[ExternalUrl]=None):  # noqa: E501
        """CommonConfiguration - a model defined in Swagger

        :param external_urls: The external_urls of this CommonConfiguration.  # noqa: E501
        :type external_urls: List[ExternalUrl]
        """
        self.swagger_types = {
            'external_urls': List[ExternalUrl]
        }

        self.attribute_map = {
            'external_urls': 'externalUrls'
        }

        self._external_urls = external_urls

    @classmethod
    def from_dict(cls, dikt) -> 'CommonConfiguration':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The CommonConfiguration of this CommonConfiguration.  # noqa: E501
        :rtype: CommonConfiguration
        """
        return util.deserialize_model(dikt, cls)

    @property
    def external_urls(self) -> List[ExternalUrl]:
        """Gets the external_urls of this CommonConfiguration.

        The collection of external urls, for example: metrics, edge, service catalog and so on  # noqa: E501

        :return: The external_urls of this CommonConfiguration.
        :rtype: List[ExternalUrl]
        """
        return self._external_urls

    @external_urls.setter
    def external_urls(self, external_urls: List[ExternalUrl]):
        """Sets the external_urls of this CommonConfiguration.

        The collection of external urls, for example: metrics, edge, service catalog and so on  # noqa: E501

        :param external_urls: The external_urls of this CommonConfiguration.
        :type external_urls: List[ExternalUrl]
        """

        self._external_urls = external_urls
