# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models import util


class TrainingConfiguration(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, metric_url: str=None):  # noqa: E501
        """TrainingConfiguration - a model defined in Swagger

        :param metric_url: The metric_url of this TrainingConfiguration.  # noqa: E501
        :type metric_url: str
        """
        self.swagger_types = {
            'metric_url': str
        }

        self.attribute_map = {
            'metric_url': 'metricUrl'
        }

        self._metric_url = metric_url

    @classmethod
    def from_dict(cls, dikt) -> 'TrainingConfiguration':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The TrainingConfiguration of this TrainingConfiguration.  # noqa: E501
        :rtype: TrainingConfiguration
        """
        return util.deserialize_model(dikt, cls)

    @property
    def metric_url(self) -> str:
        """Gets the metric_url of this TrainingConfiguration.


        :return: The metric_url of this TrainingConfiguration.
        :rtype: str
        """
        return self._metric_url

    @metric_url.setter
    def metric_url(self, metric_url: str):
        """Sets the metric_url of this TrainingConfiguration.


        :param metric_url: The metric_url of this TrainingConfiguration.
        :type metric_url: str
        """

        self._metric_url = metric_url
